@@ -1,5 +1,5 @@
 #include "covariance.h"
-
+#include "chol_cache.h"
 // Definition:
 //
 // Y_i = X_i * beta + epsilon_i, i = 1, ..., n_subjects
@@ -31,7 +31,6 @@ Type objective_function<Type>::operator() ()
   DATA_MATRIX(x_matrix);           // Model matrix (dimension n x p).
   DATA_VECTOR(y_vector);           // Response vector (length n).
   DATA_VECTOR(weights_vector);     // Weights vector (length n).
-  DATA_IVECTOR(visits_zero_inds);  // Zero-based Visits vector (length n).
   DATA_MATRIX(coordinates);        // Coordinates matrix.
   DATA_INTEGER(n_visits);          // Number of visits, which is the dimension of the covariance matrix.
   DATA_INTEGER(n_subjects);        // Number of subjects.
@@ -55,52 +54,32 @@ Type objective_function<Type>::operator() ()
   vector<Type> y_vec_tilde = vector<Type>::Zero(y_vector.rows());
   // Sum of the log determinant will be incrementally calculated here.
   Type sum_log_det = 0.0;
-  // Get number of variance parameters for one group.
-  int theta_one_group_size = theta.size() / n_groups;
+
   // Convert is_spatial_int to bool.
   bool is_spatial = (is_spatial_int == 1);
-  // Create the lower triangular Cholesky factor of the visit x visit covariance matrix.
-  int dim_cov_mat;
-  if (is_spatial) {
-    dim_cov_mat = 2;
-  } else {
-    dim_cov_mat = n_visits;
-  }
-  matrix<Type> covariance_lower_chol = get_cov_lower_chol_grouped(theta, dim_cov_mat, cov_type, n_groups, is_spatial);
+  // Diagonal of weighted covariance
+  vector<Type> diag_cov_inv_sqrt(x_matrix.rows());
+  // Cholesky group object
+  auto chols_group = chol_cache_groups<Type>(theta, n_groups, is_spatial, cov_type, n_visits);
   // Go through all subjects and calculate quantities initialized above.
   for (int i = 0; i < n_subjects; i++) {
     // Start index and number of visits for this subject.
     int start_i = subject_zero_inds(i);
     int n_visits_i = subject_n_visits(i);
-    // Obtain Cholesky factor Li.
-    matrix<Type> Li;
+    std::vector<int> visit_i(n_visits_i);
+    matrix<Type> dist_i(n_visits_i, n_visits_i);
     if (!is_spatial) {
-      matrix<Type> lower_chol = covariance_lower_chol.block(subject_groups(i) * n_visits, 0,  n_visits, n_visits);
-      if (n_visits_i < n_visits) {
-        // This subject has less visits, therefore we need to recalculate the Cholesky factor.
-        vector<int> visits_i = visits_zero_inds.segment(start_i, n_visits_i);
-        Eigen::SparseMatrix<Type> sel_mat = get_select_matrix<Type>(visits_i, n_visits);
-        // Note: We cannot use triangular view for covariance_lower_chol here because sel_mat is sparse.
-        matrix<Type> Ltildei = sel_mat * lower_chol;
-        matrix<Type> cov_i = tcrossprod(Ltildei);
-        Eigen::LLT<Eigen::Matrix<Type,Eigen::Dynamic,Eigen::Dynamic> > cov_i_chol(cov_i);
-        Li = cov_i_chol.matrixL();
-      } else {
-        // This subject has full number of visits, therefore we can just take the original factor.
-        Li = lower_chol;
+      for (int j = 0; j < n_visits_i; j++) {
+        visit_i[j] = int(asDouble(coordinates(start_i + j, 0)));
       }
     } else {
-      matrix<Type> distance_i = euclidean(matrix<Type>(coordinates.block(start_i, 0, n_visits_i, coordinates.cols())));
-      // Obtain Cholesky factor Li.
-      vector<Type> theta_i = theta.segment(subject_groups(i) * theta_one_group_size, theta_one_group_size);
-      Li = get_spatial_covariance_lower_chol(theta_i, distance_i, cov_type);
+      dist_i = euclidean(matrix<Type>(coordinates.block(start_i, 0, n_visits_i, coordinates.cols())));
     }
-
-
+    // Obtain Cholesky factor Li.
+    matrix<Type> Li = chols_group.cache[subject_groups[i]]->get_chol(visit_i, dist_i);
     // Calculate weighted Cholesky factor for this subject.
     Eigen::DiagonalMatrix<Type,Eigen::Dynamic,Eigen::Dynamic> Gi_inv_sqrt = weights_vector.segment(start_i, n_visits_i).cwiseInverse().sqrt().matrix().asDiagonal();
     Li = Gi_inv_sqrt * Li;
-
     // Calculate scaled design matrix and response vector for this subject.
     matrix<Type> Xi = x_matrix.block(start_i, 0, n_visits_i, x_matrix.cols());
     matrix<Type> XiTilde = Li.template triangularView<Eigen::Lower>().solve(Xi);
@@ -113,7 +92,8 @@ Type objective_function<Type>::operator() ()
     XtWY += XiTilde.transpose() * YiTilde;
     vector<Type> LiDiag = Li.diagonal();
     sum_log_det += sum(log(LiDiag));
-
+    // Cache the reciprocal of square root of diagonal of covariance
+    diag_cov_inv_sqrt.segment(start_i, n_visits_i) = vector<Type>(tcrossprod(Li).diagonal()).rsqrt();
     // Save stuff.
     x_mat_tilde.block(start_i, 0, n_visits_i, x_matrix.cols()) = XiTilde;
     y_vec_tilde.segment(start_i, n_visits_i) = YiTilde.col(0);
@@ -131,9 +111,11 @@ Type objective_function<Type>::operator() ()
   // Calculate negative log-likelihood.
   Type neg_log_lik;
 
+  // Always extract the D vector since we want to report this below.
+  vector<Type> XtWX_D = XtWX_decomposition.vectorD();
+
   if (reml == 1) {
     // Use restricted maximum likelihood.
-    vector<Type> XtWX_D = XtWX_decomposition.vectorD();
     Type XtWX_log_det = XtWX_D.log().sum();
     neg_log_lik = (x_matrix.rows() - x_matrix.cols()) / 2.0 * log(2.0 * M_PI) +
       sum_log_det +
@@ -156,6 +138,17 @@ Type objective_function<Type>::operator() ()
   matrix<Type> beta_vcov = XtWX_decomposition.solve(Identity);
   REPORT(beta_vcov);
 
+  // Also return the decomposition components L and D.
+  matrix<Type> XtWX_L(XtWX.rows(), XtWX.cols());
+  XtWX_L = XtWX_decomposition.matrixL();
+  REPORT(XtWX_L);
+  REPORT(XtWX_D);
+
+  // normalized residual
+  REPORT(epsilonTilde);
+  // inverse square root of diagonal of covariance
+  REPORT(diag_cov_inv_sqrt);
+  matrix<Type> covariance_lower_chol = chols_group.get_default_chol();
   REPORT(covariance_lower_chol);
 
   return neg_log_lik;
